from datetime import date

from django.contrib import admin
from django.utils.translation import gettext_lazy as _


class DeletedListFilter(admin.SimpleListFilter):
    title = _('是否删除')
    parameter_name = 'deleted'

    def lookups(self, request, model_admin):
        return (
            ('deleted', _('是')),
            ('using', _('否')),
        )

    def queryset(self, request, queryset):

        if self.value() == 'deleted':
            # todo 暂未找到更改manager的方法,先使用此方法
            # return queryset.model.all_objects.filter(delete_time__gt=0)
            return queryset.filter(delete_time__gt=0)
        if self.value() == 'using':
            # return queryset.model.all_objects.filter(delete_time=0)
            return queryset.filter(delete_time=0)



from datetime import datetime
from enum import Enum, EnumMeta
from django.db import models
from simple_common.local import local
from django.conf import settings


class ModelBase(models.Model):
    date_create = models.DateTimeField(auto_now_add=True)
    date_modify = models.DateTimeField(auto_now=True)

    user_create = models.ForeignKey(to=settings.AUTH_USER_MODEL, on_delete=models.PROTECT,
                                    related_name="%(app_label)s_%(class)s_user_create",
                                    related_query_name="%(app_label)s_%(class)s_user_create",
                                    null=True, blank=True)

    user_modify = models.ForeignKey(to=settings.AUTH_USER_MODEL, on_delete=models.PROTECT,
                                    related_name="%(app_label)s_%(class)s_user_modify",
                                    related_query_name="%(app_label)s_%(class)s_user_modify",
                                    null=True, blank=True)

    def save(self, *args, **kwargs):
        if not self.user_create:
            if hasattr(local, 'user') and local.user.is_authenticated:
                self.user_create = local.user

        if not self.user_modify:
            if hasattr(local, 'user') and local.user.is_authenticated:
                self.user_modify = local.user

        if kwargs.get('update_modify_user', False):
            self.user_modify = local.user

        super().save(*args, **kwargs)

    class Meta:
        abstract = True
        ordering = ['-date_create']


class FilterDeleteManager(models.Manager):

    def get_queryset(self):
        return super().get_queryset().order_by('-date_create')

    def filter_used(self):
        return self.get_queryset().filter(delete_time=0)

    def filter_deleted(self):
        return self.get_queryset().filter(delete_time__gt=0)


class DModelBase(ModelBase):
    delete_time = models.IntegerField(default=0)
    user_delete = models.ForeignKey(to=settings.AUTH_USER_MODEL, on_delete=models.PROTECT,
                                    related_name="%(app_label)s_%(class)s_user_delete",
                                    related_query_name="%(app_label)s_%(class)s_user_delete",
                                    null=True, blank=True)

    objects = FilterDeleteManager()

    def delete(self, using=None, keep_parents=False, delete=True):
        if delete:
            return super().delete()
        else:
            now = datetime.now()
            self.delete_time = int(now.timestamp())
            self.save(update_fields=['delete_time'])
            # todo return value

    def real_delete(self, using=None, keep_parents=False):
        return self.delete(using, keep_parents, delete=True)

    def fake_delete(self, using=None, keep_parents=False):
        return self.delete(using, keep_parents, delete=False)

    class Meta:
        abstract = True
        ordering = ['-date_create']


class ChoiceEnumMeta(EnumMeta):
    def __iter__(self):
        return ((tag.name, tag.value) for tag in super().__iter__())


class ScopeTypeChoice(Enum, metaclass=ChoiceEnumMeta):
    mine_scope = 'my scope'
    mine_public_scope = 'mine can share scope'
    department_scope = 'department scope'
    department_public_scope = 'share in department'
    global_scope = 'global scope'

    def __call__(self, *args, **kwargs):
        return self.name



from django.utils.deprecation import MiddlewareMixin

from simple_common.local import local


class LocalMiddleware(MiddlewareMixin):

    def process_request(self, request):
        self.set_local(request)

    def set_local(self, request):
        user = getattr(request, 'user', None)
        local.user = user



from django.contrib import admin
from django.contrib.admin.models import LogEntry
from django.utils.translation import gettext_lazy as _
from django.contrib.auth.models import Permission, ContentType
from django.contrib.auth.admin import UserAdmin
from django.contrib.sessions.models import Session
from django_app_base.models import *


@admin.register(User)
class MUserAdmin(UserAdmin):
    fieldsets = (
        (None, {'fields': ('username', 'password')}),
        (_('Personal info'), {'fields': ('name', 'phone', 'email')}),
        (_('Permissions'), {'fields': ('is_active', 'is_staff', 'is_superuser',
                                       'groups', 'user_permissions')}),
        (_('Important dates'), {'fields': ('last_login', 'date_joined')}),
    )
    filter_horizontal = ('groups', 'user_permissions')

    list_display = ('id', 'username', 'name', 'phone', 'email', 'is_staff', 'is_active')


@admin.register(Session)
class SessionAdmin(admin.ModelAdmin):
    list_display = ('session_key', 'expire_date')


@admin.register(ContentType)
class ContentTypeAdmin(admin.ModelAdmin):
    list_display = ('app_label', 'model')
    search_fields = ('app_label', 'model')


@admin.register(Permission)
class PermissionAdmin(admin.ModelAdmin):

    def app_model(self, obj):
        return "[%s] %s" % (obj.content_type.app_label, obj.content_type.name)

    app_model.short_description = 'app_model'

    list_display = ('name', 'codename', 'content_type', 'app_model')
    search_fields = ('name', 'codename')


@admin.register(LogEntry)
class LogEntryAdmin(admin.ModelAdmin):
    list_display = ('id', 'action_time', 'action_flag', 'user', 'content_type',
                    'object_repr')



from abc import ABC
from django.conf import settings
from django.utils.module_loading import import_string


# DEFAULTS = {
# }
#
# IMPORT_STRINGS = [
#
# ]


def perform_import(val, setting_name):
    if val is None:
        return None
    elif isinstance(val, str):
        return import_from_string(val, setting_name)
    elif isinstance(val, (list, tuple)):
        return [import_from_string(item, setting_name) for item in val]
    return val


def import_from_string(val, setting_name):
    try:
        return import_string(val)
    except ImportError as e:
        msg = "Could not import '%s' for setting '%s'. %s: %s." % (val, setting_name, e.__class__.__name__, e)
        raise ImportError(msg)


class CustomSettings(ABC):

    def __init__(self, base_name, defaults, import_strings, user_settings=None):
        self._base_name = base_name
        if user_settings:
            self._user_settings = user_settings
        self.defaults = defaults
        self.import_strings = import_strings
        self._cached_attrs = set()

    @property
    def user_settings(self):
        if not hasattr(self, '_user_settings'):
            self._user_settings = getattr(settings, self._base_name, {})
        return self._user_settings

    def __getattr__(self, attr):
        if attr not in self.defaults:
            raise AttributeError("Invalid API setting: '%s'" % attr)

        try:
            val = self.user_settings[attr]
        except KeyError:
            val = self.defaults[attr]

        if attr in self.import_strings:
            val = perform_import(val, attr)

        self._cached_attrs.add(attr)
        setattr(self, attr, val)
        return val

    def reload(self):
        for attr in self._cached_attrs:
            delattr(self, attr)
        self._cached_attrs.clear()
        if hasattr(self, '_user_settings'):
            delattr(self, '_user_settings')



import logging

from django.middleware.csrf import CsrfViewMiddleware
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth import get_user_model

from rest_framework import exceptions

logger = logging.getLogger()


def get_authorization_header(request):
    auth = request.META.get('HTTP_AUTHORIZATION', b'')
    return auth


class CSRFCheck(CsrfViewMiddleware):
    def _reject(self, request, reason):
        # Return the failure reason instead of an HttpResponse
        return reason


class BaseAuthentication(object):

    def authenticate(self, request):
        raise NotImplementedError(".authenticate() must be overridden.")

    def authenticate_header(self, request):
        pass


class TokenAuthentication(BaseAuthentication):
    """
    Simple token based authentication.

    Clients should authenticate by passing the token key in the "Authorization"
    HTTP header, prepended with the string "Token ".  For example:

        Authorization: Token 401f7ac837da42b97f613d789819ff93537bee6a
    """

    keyword = 'Token'
    model = None

    def get_model(self):
        if self.model is not None:
            return self.model
        from rest_framework.authtoken.models import Token
        return Token

    """
    A custom token model may be used, but must have the following properties.

    * key -- The string identifying the token
    * user -- The user to which the token belongs
    """

    def authenticate(self, request):
        auth = get_authorization_header(request).split()

        if not auth or auth[0].lower() != self.keyword.lower():
            return None

        if len(auth) == 1:
            msg = _('Invalid token header. No credentials provided.')
            raise exceptions.AuthenticationFailed(msg)
        elif len(auth) > 2:
            msg = _('Invalid token header. Token string should not contain spaces.')
            raise exceptions.AuthenticationFailed(msg)

        try:
            token = auth[1]
        except UnicodeError:
            msg = _('Invalid token header. Token string should not contain invalid characters.')
            raise exceptions.AuthenticationFailed(msg)
        try:
            user = self.authenticate_credentials(token)
            request.session['is_api_token_request'] = True
            return user
        except Exception:
            return None

    def authenticate_credentials(self, key):
        model = self.get_model()
        try:
            token = model.objects.select_related('user').get(key=key)
        except model.DoesNotExist:
            logger.error('Invalid token.')
            raise exceptions.AuthenticationFailed(_('Invalid token.'))

        if not token.user.is_active:
            raise exceptions.AuthenticationFailed(_('User inactive or deleted.'))

        return token.user

    def authenticate_header(self, request):
        return self.keyword

    def get_user(self, user_id):
        try:
            user = get_user_model()._default_manager.get(pk=user_id)
        except get_user_model().DoesNotExist:
            return None
        return user if self.user_can_authenticate(user) else None

    def user_can_authenticate(self, user):
        """
        Reject users with is_active=False. Custom user models that don't have
        that attribute are allowed.
        """
        is_active = getattr(user, 'is_active', None)
        return is_active or is_active is None



import logging
from django.utils.deprecation import MiddlewareMixin

logger = logging.getLogger()


class AuthAclMiddleware(MiddlewareMixin):

    def process_response(self, request, response):
        try:
            if request.session.get("is_api_token_request", False):
                request.session.flush()
        except Exception as e:
            logger.error('clear session middleware failed', exc_info=e)
        return response



import json
from rest_framework.request import Request
from django.http import JsonResponse
from django.http.request import HttpRequest as DjangoHttpRequest
from django.http import QueryDict

result = dict()
result_arr = []


def get_params(request, *args, **kwargs):
    """
    django rest framework 的request 多了一个data,各种请求方式的参数都会在里面
    :param request:
    :param args:
    :param kwargs:
    :return:
    """

    content_type = getattr(request, 'content_type', None)
    method = request.method

    # django request
    if isinstance(request, DjangoHttpRequest):
        if method == 'GET':
            return request.GET
        if method == 'POST':
            if content_type == 'application/json':
                return json.loads(request.body.decode())
            else:
                return request.POST

    # django rest framework request
    if isinstance(request, Request):

        query_params = request.query_params
        if isinstance(query_params, QueryDict):
            query_params = query_params.dict()
        result_data = request.data
        if isinstance(result_data, QueryDict):
            result_data = result_data.dict()

        if query_params != {}:
            return query_params
        else:
            return result_data


class SuccessJsonResponse(JsonResponse):

    def __init__(self, data):
        _data = dict(data=data,
                     error=None,
                     success=True,
                     error_msg=None)
        super().__init__(data=_data)



from rest_framework import generics
from rest_framework.response import Response


class BaseViewMixin(generics.GenericAPIView):
    def get_serializer_date(self, request):
        serializer_class = self.get_serializer_class()
        serializer = serializer_class(data=request.data)
        serializer.is_valid(raise_exception=True)
        return serializer.data

    def get_list_serializer(self, *args, **kwargs):
        if getattr(self, 'list_serializer_class', None):
            serializer_class = self.list_serializer_class
        else:
            serializer_class = self.get_serializer_class()
        kwargs['context'] = self.get_serializer_context()
        return serializer_class(*args, **kwargs)

    def list(self, request, *args, **kwargs):
        queryset = self.filter_queryset(self.get_queryset())
        page = self.paginate_queryset(queryset)
        if page is not None:
            serializer = self.get_list_serializer(page, many=True)
            return self.get_paginated_response(serializer.data)

        serializer = self.get_list_serializer(queryset, many=True)
        return Response(serializer.data)



from django.contrib.auth.mixins import UserPassesTestMixin
from rest_framework import generics


class AdminUserRequiredMixin(UserPassesTestMixin):
    def test_func(self):
        if not self.request.user.is_authenticated:
            return False
        elif not self.request.user.is_superuser:
            self.raise_exception = True
            return False
        return True



from django.utils.deprecation import MiddlewareMixin

from simple_common.utils.string_utils import str_contain_some


class CommonMiddleware(MiddlewareMixin):
    def process_request(self, request):

        setattr(request, '_dont_enforce_csrf_checks', True)

        if self.is_from_browser(request):
            setattr(request, 'origin_type', 'browser')
        else:
            setattr(request, 'origin_type', 'api')

    @staticmethod
    def is_from_browser(request):
        user_agent = request.META.get('HTTP_USER_AGENT', None)
        if user_agent:
            return str_contain_some(user_agent.lower(), 'mozilla', 'applewebkit', 'chrome', 'safari')
        else:
            return False



from django_app_base.custom_settings import CustomSettings

DEFAULTS = {
    'AUTH_PERMIT_PATH_LIST': [],
    'DEV_ENV': True,
    'AUTH_ACL': True
}

IMPORT_STRINGS = [

]


class DjangoAppBaseSettings(CustomSettings):
    pass


app_base_settings = DjangoAppBaseSettings(base_name='APP_BASE',
                                          defaults=DEFAULTS,
                                          import_strings=IMPORT_STRINGS)



from django.http import JsonResponse


class SuccessJsonResponse(JsonResponse):

    def __init__(self, data):
        _data = dict(data=data,
                     error=None,
                     success=True,
                     error_msg=None)
        super().__init__(data=_data)



from django.db import models
from django.contrib.auth.models import AbstractUser
from django.utils.translation import ugettext_lazy as _

__all__ = ['User']


class User(AbstractUser):
    username = models.CharField(
        max_length=128, unique=True, verbose_name=_('Username')
    )

    name = models.CharField(max_length=128, verbose_name=_('Name'))

    phone = models.CharField(
        max_length=20, blank=True, null=True, verbose_name=_('Phone')
    )

    def save(self, *args, **kwargs):
        update_fields = kwargs.get('update_fields', None)
        if len(kwargs.keys()) == 1 and update_fields:
            try:
                super().save(*args, **kwargs)
            except Exception as e:
                # 当数据库只读时此字段无法修改,但是让用户可以登录是合理的
                print(e)
                pass
        else:
            super().save(*args, **kwargs)

    def has_role(self, role_name):
        return role_name in [value[0] for value in self.roles.all().values_list('name')]

    def get_all_role_names(self):
        return [role.name for role in self.role_set.all()]



from collections import OrderedDict, namedtuple

from rest_framework.pagination import PageNumberPagination
from rest_framework.exceptions import NotFound
from rest_framework.response import Response
from rest_framework.compat import coreapi, coreschema
from rest_framework.utils.urls import remove_query_param, replace_query_param
from django.core.paginator import Paginator as DjangoPaginator
from django.core.paginator import InvalidPage
from django.template import loader
from django.utils.encoding import force_text
from django.utils.translation import ugettext_lazy as _


class StandardResultsSetPagination(PageNumberPagination):
    page_size = 10
    page_size_query_param = 'page_size'
    max_page_size = 100


##################################################################################

Cursor = namedtuple('Cursor', ['offset', 'reverse', 'position'])
PageLink = namedtuple('PageLink', ['url', 'number', 'is_active', 'is_break'])

PAGE_BREAK = PageLink(url=None, number=None, is_active=False, is_break=True)


def _positive_int(integer_string, strict=False, cutoff=None):
    """
    Cast a string to a strictly positive integer.
    """
    ret = int(integer_string)
    if ret < 0 or (ret == 0 and strict):
        raise ValueError()
    if cutoff:
        return min(ret, cutoff)
    return ret


def _get_displayed_page_numbers(current, final):
    """
    This utility function determines a list of page numbers to display.
    This gives us a nice contextually relevant set of page numbers.

    For example:
    current=14, final=16 -> [1, None, 13, 14, 15, 16]

    This implementation gives one page to each side of the cursor,
    or two pages to the side when the cursor is at the edge, then
    ensures that any breaks between non-continuous page numbers never
    remove only a single page.

    For an alternative implementation which gives two pages to each side of
    the cursor, eg. as in GitHub issue list pagination, see:

    https://gist.github.com/tomchristie/321140cebb1c4a558b15
    """
    assert current >= 1
    assert final >= current

    if final <= 5:
        return list(range(1, final + 1))

    # We always include the first two pages, last two pages, and
    # two pages either side of the current page.
    included = {1, current - 1, current, current + 1, final}

    # If the break would only exclude a single page number then we
    # may as well include the page number instead of the break.
    if current <= 4:
        included.add(2)
        included.add(3)
    if current >= final - 3:
        included.add(final - 1)
        included.add(final - 2)

    # Now sort the page numbers and drop anything outside the limits.
    included = [
        idx for idx in sorted(list(included))
        if 0 < idx <= final
    ]

    # Finally insert any `...` breaks
    if current > 4:
        included.insert(1, None)
    if current < final - 3:
        included.insert(len(included) - 1, None)
    return included


def _get_page_links(page_numbers, current, url_func):
    """
    Given a list of page numbers and `None` page breaks,
    return a list of `PageLink` objects.
    """
    page_links = []
    for page_number in page_numbers:
        if page_number is None:
            page_link = PAGE_BREAK
        else:
            page_link = PageLink(
                url=url_func(page_number),
                number=page_number,
                is_active=(page_number == current),
                is_break=False
            )
        page_links.append(page_link)
    return page_links


class OncePageNumberPagination(object):
    """
    A simple page number based style that supports page numbers as
    query parameters. For example:

    http://api.example.org/accounts/?page=4
    http://api.example.org/accounts/?page=4&page_size=100
    """
    # The default page size.
    # Defaults to `None`, meaning pagination is disabled.
    page_size = 15

    django_paginator_class = DjangoPaginator

    # Client can control the page using this query parameter.
    page_query_param = 'page'
    page_query_description = _('A page number within the paginated result set.')

    # Client can control the page size using this query parameter.
    # Default is 'None'. Set to eg 'page_size' to enable usage.
    page_size_query_param = None
    page_size_query_description = _('Number of results to return per page.')

    # Set to an integer to limit the maximum page size the client may request.
    # Only relevant if 'page_size_query_param' has also been set.
    max_page_size = None

    last_page_strings = ('last',)

    template = 'rest_framework/pagination/numbers.html'

    invalid_page_message = _('Invalid page.')

    def __init__(self, page_size):
        self.page_size = page_size

    def paginate_queryset(self, queryset, request, view=None):
        """
        Paginate a queryset if required, either returning a
        page object, or `None` if pagination is not configured for this view.
        """
        page_size = self.get_page_size(request)
        if not page_size:
            return None

        paginator = self.django_paginator_class(queryset, page_size)
        page_number = request.query_params.get(self.page_query_param, 1)
        if page_number in self.last_page_strings:
            page_number = paginator.num_pages

        try:
            self.page = paginator.page(page_number)
        except InvalidPage as exc:
            msg = self.invalid_page_message.format(
                page_number=page_number, message=str(exc)
            )
            raise NotFound(msg)

        if paginator.num_pages > 1 and self.template is not None:
            # The browsable API should display pagination controls.
            self.display_page_controls = True

        self.request = request
        return list(self.page)

    def get_paginated_response(self, data):
        return Response(OrderedDict([
            ('count', self.page.paginator.count),
            ('next', self.get_next_link()),
            ('previous', self.get_previous_link()),
            ('results', data)
        ]))

    def get_page_size(self, request):
        if self.page_size_query_param:
            try:
                return _positive_int(
                    request.query_params[self.page_size_query_param],
                    strict=True,
                    cutoff=self.max_page_size
                )
            except (KeyError, ValueError):
                pass

        return self.page_size

    def get_next_link(self):
        if not self.page.has_next():
            return None
        url = self.request.build_absolute_uri()
        page_number = self.page.next_page_number()
        return replace_query_param(url, self.page_query_param, page_number)

    def get_previous_link(self):
        if not self.page.has_previous():
            return None
        url = self.request.build_absolute_uri()
        page_number = self.page.previous_page_number()
        if page_number == 1:
            return remove_query_param(url, self.page_query_param)
        return replace_query_param(url, self.page_query_param, page_number)

    def get_html_context(self):
        base_url = self.request.build_absolute_uri()

        def page_number_to_url(page_number):
            if page_number == 1:
                return remove_query_param(base_url, self.page_query_param)
            else:
                return replace_query_param(base_url, self.page_query_param, page_number)

        current = self.page.number
        final = self.page.paginator.num_pages
        page_numbers = _get_displayed_page_numbers(current, final)
        page_links = _get_page_links(page_numbers, current, page_number_to_url)

        return {
            'previous_url': self.get_previous_link(),
            'next_url': self.get_next_link(),
            'page_links': page_links
        }

    def to_html(self):
        template = loader.get_template(self.template)
        context = self.get_html_context()
        return template.render(context)

    def get_schema_fields(self, view):
        assert coreapi is not None, 'coreapi must be installed to use `get_schema_fields()`'
        assert coreschema is not None, 'coreschema must be installed to use `get_schema_fields()`'
        fields = [
            coreapi.Field(
                name=self.page_query_param,
                required=False,
                location='query',
                schema=coreschema.Integer(
                    title='Page',
                    description=force_text(self.page_query_description)
                )
            )
        ]
        if self.page_size_query_param is not None:
            fields.append(
                coreapi.Field(
                    name=self.page_size_query_param,
                    required=False,
                    location='query',
                    schema=coreschema.Integer(
                        title='Page size',
                        description=force_text(self.page_size_query_description)
                    )
                )
            )
        return fields



from django.utils.deprecation import MiddlewareMixin
from django.contrib.auth import authenticate, login

from django.views.decorators.csrf import csrf_exempt


class AuthenticationMiddleware(MiddlewareMixin):

    @csrf_exempt
    def process_request(self, request):
        # after django auth, user may be anonymous
        if getattr(request, 'user', None):
            if request.user.is_authenticated:
                pass
            else:
                if request.META.get("HTTP_AUTHORIZATION", None):
                    user = authenticate(request)
                    if user:
                        login(request, user)
                    else:
                        pass
        else:
            if request.META.get("HTTP_AUTHORIZATION", None):
                user = authenticate(request)
                if user:
                    login(request, user)
                else:
                    pass
            else:
                pass



import logging
from django.utils.deprecation import MiddlewareMixin
from django.http import HttpResponseForbidden
from django_app_base.settings import app_base_settings

logger = logging.getLogger()


class PermitUserMiddleware(MiddlewareMixin):

    def process_request(self, request):
        if app_base_settings.DEV_ENV == "dev" and not app_base_settings.AUTH_ACL:
            pass
        elif self.in_white_list(request):
            pass
        elif request.user.is_superuser:
            pass
        else:
            flag = False
            for group in request.user.groups.all():
                if group.name == 'white_user':
                    flag = True
                    break
            if flag:
                pass
            else:
                logger.info('WhiteUserMiddleware Forbidden!')
                return HttpResponseForbidden()

    @staticmethod
    def in_white_list(request):
        for white_path in app_base_settings.AUTH_PERMIT_PATH_LIST:
            if request.path.startswith(white_path):
                return True

        return False



from rest_framework import serializers


class NullSerializer(serializers.Serializer):
    def create(self, validated_data):
        pass

    def update(self, instance, validated_data):
        pass



import logging
from django.contrib.auth import authenticate, login
from django.utils.deprecation import MiddlewareMixin
from django.http import HttpResponseForbidden
from django.shortcuts import redirect
from django.conf import settings
from django_app_base.settings import app_base_settings

logger = logging.getLogger()


class MustLoginMiddleware(MiddlewareMixin):

    def process_request(self, request):

        if app_base_settings.DEV_ENV == "dev" and not app_base_settings.AUTH_ACL:
            pass
        elif self.in_white_list(request):
            pass
        elif not request.user.is_authenticated:
            user = authenticate(request)
            if user:
                login(request, user)
            else:
                return self.forbidden(request)

    @staticmethod
    def forbidden(request):
        if request.is_ajax():
            logger.debug('no auth,request forbidden,request is ajax')
            return HttpResponseForbidden()
        else:
            logger.debug('no auth,request forbidden,request is not ajax')
            # return redirect('%s?next=%s' % (settings.LOGIN_URL, request.path))
            return redirect(settings.LOGIN_URL)

    @staticmethod
    def in_white_list(request):
        for white_path in app_base_settings.AUTH_PERMIT_PATH_LIST:
            if request.path.startswith(white_path):
                return True

        return False



from django_app_base.middleware.authentication import AuthenticationMiddleware
from django_app_base.middleware.common import CommonMiddleware
from django_app_base.middleware.must_login import MustLoginMiddleware
from django_app_base.middleware.local import LocalMiddleware
from django_app_base.middleware.permit_user_middleware import PermitUserMiddleware



from simple_common.local import local
from collections import defaultdict


class RoleService(object):

    @staticmethod
    def get_perm_names_on_user(user):
        """
        从django的permission表中获取所有的permission
        :param user:
        :return:
        """
        perms = user.user_permissions.all().values_list('content_type__app_label', 'codename', 'name').order_by()
        return [["%s.%s" % (ct, codename), name] for ct, codename, name in perms]

    @staticmethod
    def get_perm_names_on_group(group):
        """
        从django的group获取相关的permission
        :param group:
        :return:
        """
        perms = group.permissions.all().values_list('content_type__app_label', 'codename', 'name').order_by()
        return [["%s.%s" % (ct, codename), name] for ct, codename, name in perms]

    @staticmethod
    def get_perm_names_on_role(role):
        """
        获取某个role的permission
        :param role:
        :return:
        """
        perms = role.permissions.all().values_list('content_type__app_label', 'codename', 'name').order_by()
        return [["%s.%s" % (ct, codename), name] for ct, codename, name in perms]

    @staticmethod
    def get_perm_names_on_department_role(department):
        """
        获取某个department的permission
        一个department可能有多个role
        :param department:
        :return:
        """
        pass

    @staticmethod
    def my_tree():
        user = local.user
        user_perms = RoleService.get_perm_names_on_user(user)
        permission_map = dict(
            user=list(user_perms),
            group=dict(),
            role=dict(),
            department=defaultdict(dict),
        )

        for role in user.role_set.all():
            # 获取每个role上面的permission
            permission_map['role'][role.name] = list(RoleService.get_perm_names_on_role(role))

        for group in user.groups.all():
            permission_map['group'][group.name] = list(RoleService.get_perm_names_on_role(group))

        if user.department:
            for role in user.department.roles.all():
                permission_map['department'][user.department.name][role.name] = list(
                    RoleService.get_perm_names_on_role(role))

        return permission_map



import logging
import uuid
from functools import wraps

logger = logging.getLogger('api')


class ServiceResp(object):
    def __init__(self, data, error, success, **kwargs):
        self.uid = uuid.uuid4().hex
        self.data = data
        self.error = error
        self.success = success
        # status_code 默认为-1, 表明未设置具体的状态码
        self.status_code = kwargs.get('status_code', -1)
        self.operation_name = kwargs.get('operation_name', None)
        self.error_msg = kwargs.get('error_msg', None)

    def get_msg(self):
        return self.error_msg

    def get_dict(self):
        return dict(data=self.data,
                    error=str(self.error),
                    success=self.success,
                    status_code=self.status_code,
                    error_msg=self.error_msg)

    def json(self):
        if hasattr(self.data, 'as_json'):
            return dict(data=self.data.as_json(),
                        error=str(self.error),
                        success=self.success,
                        status_code=self.status_code,
                        error_msg=self.error_msg)
        else:
            return dict(data=self.data,
                        error=str(self.error),
                        success=self.success,
                        status_code=self.status_code,
                        error_msg=self.error_msg)


class SuccessResp(ServiceResp):
    def __init__(self, data, **kwargs):
        super().__init__(data=data, error=None, success=True, **kwargs)


class ErrorResp(ServiceResp):
    def __init__(self, error, **kwargs):
        super().__init__(data=None, error=error, success=False, **kwargs)


def service_wrap(f):
    @wraps(f)
    def wrapper(*args, **kwargs):
        service_resp = ErrorResp(None)
        try:

            resp = f(*args, **kwargs)
            if isinstance(resp, ServiceResp):
                service_resp = resp
            elif isinstance(resp, ErrorResp):
                # error_resp 不处理
                pass
            else:
                service_resp = SuccessResp(data=resp)
        except Exception as e:
            logger.error('service wrap catch error', exc_info=True)
            if e.__dict__.get('code', None):
                service_resp = ErrorResp(e, error_msg=str(e), status_code=e.code)
            else:
                service_resp = ErrorResp(e, error_msg=str(e))
        finally:
            return service_resp

    return wrapper



from setuptools import setup
import re
import os

package = 'django_app_base'

requirements = [
    'Django>=2.0',
    'djangorestframework',
    'jsonfield'
]


def get_version():
    init_py = open(os.path.join(package, '__init__.py')).read()
    return re.search("^__version__ = ['\"]([^'\"]+)['\"]", init_py, re.MULTILINE).group(1)


def get_packages():
    return [dirpath
            for dirpath, dirnames, filenames in os.walk(package)
            if os.path.exists(os.path.join(dirpath, '__init__.py'))]


def get_package_data():
    walk = [(dirpath.replace(package + os.sep, '', 1), filenames)
            for dirpath, dirnames, filenames in os.walk(package)
            if not os.path.exists(os.path.join(dirpath, '__init__.py'))]

    filepaths = []
    for base, filenames in walk:
        filepaths.extend([os.path.join(base, filename)
                          for filename in filenames])
    return {package: filepaths}


with open("README.md", "r", encoding='UTF-8') as fh:
    long_description = fh.read()

setup(
    name='django_app_base',
    version=get_version(),
    url='https://github.com/xunull/django-user-role',
    description='A django app for user role based permissions.',
    long_description=long_description,
    long_description_content_type="text/markdown",
    author='xunull',
    packages=get_packages(),
    package_data=get_package_data(),
    install_requires=requirements,
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: Apache Software License",
        "Operating System :: OS Independent",
    ],
)



import re
import collections
from django.contrib.auth import get_user_model


def get_admin():
    return get_user_model.objects.get(username='admin')


def user_is_authenticated(user):
    if isinstance(user.is_authenticated, collections.Callable):
        authenticated = user.is_authenticated()
    else:
        authenticated = user.is_authenticated

    return authenticated



from django.http.response import HttpResponse
from rest_framework import viewsets
from rest_framework.decorators import action
from rest_framework.response import Response
from rest_framework import status
from rest_framework.settings import api_settings
